// SPDX-FileCopyrightText: Heiko Schaefer <heiko@schaefer.name>
// SPDX-License-Identifier: MIT OR Apache-2.0

use serde::{Deserialize, Serialize};

const CONFIG_APP: &str = "openpgp-card-state";
const CONFIG_FILE_NAME: &str = "config";

#[derive(Default, Serialize, Deserialize)]
pub(crate) struct Config {
    cards: Vec<CardSettings>,

    /// If set, this applies when putting a PIN for a new card
    pub(crate) default_pin_storage: Option<PinStorageType>,
}

#[derive(Clone, Serialize, Deserialize)]
pub(crate) struct CardSettings {
    pub(crate) ident: String,
    pub(crate) pin_storage: Option<PinStorage>,
    pub(crate) nickname: Option<String>,
}

/// Specifies a type of PIN storage backend, optionally combined with global configuration for
/// that backend (e.g. a path to a binary)
#[derive(Clone, Serialize, Deserialize, PartialEq)]
pub enum PinStorageType {
    /// The keyring crate is used to store and retrieve the User PIN
    Keyring,

    /// The User PIN is stored directly in this config file.
    Direct,
}

/// Specifies a type of PIN storage backend, optionally combined with configuration for that
/// backend, and/or a card-specific payload
#[derive(Clone, Serialize, Deserialize)]
pub(crate) enum PinStorage {
    /// The keyring crate is used to store and retrieve the User PIN
    Keyring,

    /// The User PIN is stored directly in this config file.
    ///
    /// An empty string signifies use of the "Direct" backend, but with no PIN currently stored
    /// (e.g. after the PIN has been dropped because it failed verification on the card)
    Direct(String),
}

impl Config {
    pub(crate) fn load() -> anyhow::Result<Self> {
        Ok(confy::load(CONFIG_APP, Some(CONFIG_FILE_NAME))?)
    }

    pub(crate) fn save(&self) -> anyhow::Result<()> {
        Ok(confy::store(CONFIG_APP, Some(CONFIG_FILE_NAME), self)?)
    }

    pub(crate) fn push(&mut self, cs: CardSettings) -> anyhow::Result<()> {
        // don't allow duplicate entries for one card ident
        if self.cards.iter().any(|old| old.ident == cs.ident) {
            return Err(anyhow::anyhow!(
                "Trying to insert duplicate entry for {}",
                cs.ident
            ));
        }

        self.cards.push(cs);
        Ok(())
    }

    pub(crate) fn remove(&mut self, ident: &str) -> anyhow::Result<()> {
        if let Some((pos, _)) = self
            .cards
            .iter()
            .enumerate()
            .find(|(_, cs)| cs.ident == ident)
        {
            self.cards.remove(pos);
        }

        Ok(())
    }

    pub(crate) fn ident(&self, ident: &str) -> Option<&CardSettings> {
        self.cards.iter().find(|card| card.ident == ident)
    }

    pub(crate) fn ident_mut(&mut self, ident: &str) -> Option<&mut CardSettings> {
        self.cards.iter_mut().find(|card| card.ident == ident)
    }
}
